


/**
 *
 * @author Christina Christodoulakis <christina at cs.toronto.edu>
 */
public class Edge {
    
    private Node startNode;
    private Node endNode;
    private Notation notation;
    
    public Edge(Node startNode,Node endNode){
        this.startNode = startNode;
        this.endNode = endNode;
    }

    
    
    /**
     * Gets the starter node
     * @return started node
     */
    public Node getStartNode() {
        return startNode;
    }

    
    /**
     * Sets the started node
     * @param startNode The node to set the starter node to
     */
    public void setStartNode(Node startNode) {
        this.startNode = startNode;
    }

    
    /**
     * Gets the end node
     * @return end node
     */
    public Node getEndNode() {
        return endNode;
    }

    
    
    /**
     * Sets the end node
     * @param endNode The node to set the end node to
     */
    public void setEndNode(Node endNode) {
        this.endNode = endNode;
    }

    
    /**
     * Get information about the nodes edge
     * @return  Information about the edge
     */
    public Notation getNotation() {
        return notation;
    }
    
    
    /**
     * Sets the edge information for the node
     * @param notation Edge information
     */
    public void setNotation(Notation notation) {
        this.notation = notation;
    }
    
    
    
    /**
     * Removes this edge/relation between two nodes
     */
    public void remove() {
        startNode.removeEdge(endNode);
    }
    
    
    
    /**
     * Restores this edge/relation between two nodes
     */
    public void restore() {
        startNode.addNode(endNode);
    }
    
    
    
    /**
     * Given one of the nodes contained in this edge, returns the OTHER node
     * @param node One of the two edges
     * @return The OTHER node, or null of the input node was not found
     */
    public Node getConnector(Node node) {
        if (node == startNode)
        {
        	return endNode;
        }
        else if (node == endNode)
        {
        	return startNode;
        }
        else
        {
        	return null;
        }
    }
    
    
    
    /**
     * Compares two edges to test if they contain the same nodes
     * @param edge The edge to test againt
     * @return True if the edges are equal, false otherwise
     */
    public boolean compare(Edge edge){
    	if ((startNode == edge.startNode) && (endNode == edge.endNode))
    	{
    		return true;
    	}
    	if ((endNode == edge.startNode) && (startNode == edge.endNode))
    	{
    		return true;
    	}
    	return false;
    }
    
}
