

import java.util.ArrayList;
import java.util.StringTokenizer;
import rita.wordnet.RiWordnet;
/**
 *
 * @author christina christodoulakis
 */
public class Util {

    public Util() {
    }

    /**
     * Given a list of nodes, looks at all names and rates 
     * the distance of words in the list
     * (rating is from [0,1])
     * 
     * looks at pairwise distances between names, and keeps lowest pairwise 
     * distance, adds them up and divides by total
     * 
     * @param nodes
     * @return distance rating
     */
    
    public float getAvgDistance(ArrayList<Node> nodes) {
   
        Integer N = nodes.size();        
        //preprocess names
        nodes = splitEntityNames(nodes);
        //Store just the list of names in an array for speed
        ArrayList<String> names = new ArrayList<String>();
        for(int i = 0; i< N; i++){
            names.add(nodes.get(i).className);
        }
        float totalDist = 0;

        for(int i=0; i<N; i++){  
            float minDist = 1;
            float dist = 0;
            String term1 = names.get(i);		
            for(int j= 0; j<N; j++){
                String term2 = names.get(j);
                if(i!=j){
                dist = compareTerms(term1, term2);
                if(dist<minDist) minDist = dist;}
            }
            totalDist = totalDist + minDist;
        }    
        return totalDist/N;
    }
    
    /**
     * For every word in term1, find distance to each word in term 2 
     * and return the shortest distance
     * @param term1 name of entity 1
     * @param term2 name of entity 2
     * @return the shortest pairwise distance between words in entity names
     */
    private float compareTerms(String term1, String term2){
        
        RiWordnet wordnet  = new RiWordnet(null); 
        StringTokenizer tokens1 = new StringTokenizer(term1, " ");         
        ArrayList<String> name1 = new ArrayList<String>();
        
        StringTokenizer tokens2 = new StringTokenizer(term2, " ");         
        ArrayList<String> name2 = new ArrayList<String>();
        
        while(tokens1.hasMoreTokens())
            name1.add(tokens1.nextToken());
        while (tokens2.hasMoreTokens())
            name2.add(tokens2.nextToken());
        
        float minDist = 1;
        // Go through all pairwise distances and return shortest
        for(int i = 0; i< name1.size(); i++){
            for(int j = 0; j<name2.size(); j++){
                float distance = wordnet.getDistance(name1.get(i), name2.get(j), "n");
                if(distance<minDist)minDist = distance;
            }            
        }               
        return minDist;
    }
/**
     * Given a list of nodes iterates through the list and renames
     * nodes (ex:   MyClass --> My Class
                    HTML --> HTML
                    PDFLoader --> PDF Loader
     * @param nodes the list of nodes
     * @return the renames list of nodes
     */
    public ArrayList<Node> splitEntityNames(ArrayList<Node> nodes) {
       
        for (int i = 0; i < nodes.size(); i++) {
            String entityName = nodes.get(i).className;
            entityName = removeUnderScore(splitCamelCase(entityName));
            nodes.get(i).className = entityName;
        }
        return nodes;
    }

  /**
     * Splits camel cased words
     * @param s the string to split
     * @return the split string
     */  static String splitCamelCase(String s) {
        return s.replaceAll(
                String.format("%s|%s|%s",
                "(?<=[A-Z])(?=[A-Z][a-z])",
                "(?<=[^A-Z])(?=[A-Z])",
                "(?<=[A-Za-z])(?=[^A-Za-z])"),
                " ");
    }
     /**
      * Replaces underscores with space
      * @param s the input string
      * @return the input string with spaces instead of underscores
      */
    static String removeUnderScore(String s){
        return s.replace("_", " ");
    }
}
